import serial


class DataComm:
    """
    此类初始化两个字典，control_button_hex_dict和normal_button_hex_dict，它们包含键盘上控制和普通按钮的十六进制值。
    如果你需要更多的按钮，请自行根据协议文档补充
    """

    def __init__(self):
        self.control_button_hex_dict = {"NULL": b'\x00',
                                        "R_WIN": b"\x80",
                                        "R_ALT": b"\x40",
                                        "R_SHIFT": b"\x20",
                                        "R_CTRL": b"\x10",
                                        "L_WIN": b"\x08",
                                        "L_ALT": b"\x04",
                                        "L_SHIFT": b"\x02",
                                        "L_CTRL": b"\x01"
                                        }
        self.normal_button_hex_dict = {"NU": b'\x00',
                                       "QQ": b"\x14",
                                       "WW": b"\x1A",
                                       "EE": b"\x08",
                                       "RR": b"\x15",
                                       "TT": b"\x17",
                                       "YY": b"\x1C",
                                       "UU": b"\x18",
                                       "II": b"\x0C",
                                       "OO": b"\x12",
                                       "PP": b"\x13",
                                       "AA": b"\x04",
                                       "SS": b"\x16",
                                       "DD": b"\x07",
                                       "FF": b"\x09",
                                       "GG": b"\x0A",
                                       "HH": b"\x0B",
                                       "JJ": b"\x0D",
                                       "KK": b"\x0E",
                                       "LL": b"\x0F",
                                       "ZZ": b"\x1D",
                                       "XX": b"\x1B",
                                       "CC": b"\x06",
                                       "VV": b"\x19",
                                       "BB": b"\x05",
                                       "NN": b"\x11",
                                       "MM": b"\x10"
                                       }

    """
    发送数据到串口。
    
    参数:
        data (str): 要发送的按键信息。
        ctrl (str): 要发送的控制键。
        port (serial): 要发送数据的串口。
    
    返回:
        bool: 如果数据成功发送，则为True，否则为False。
    """

    def send_data(self, data, ctrl='', port=serial):
        # 将字符转写为数据包
        HEAD = b'\x57\xAB'  # 帧头
        ADDR = b'\x00'  # 地址
        CMD = b'\x02'  # 命令
        LEN = b'\x08'  # 数据长度
        DATA = b''  # 数据

        # 控制键
        if ctrl == '':
            DATA += b'\x00'
        elif isinstance(ctrl, int):
            DATA += bytes([ctrl])
        else:
            DATA += self.control_button_hex_dict[ctrl]

        # DATA固定码
        DATA += b'\x00'

        # 读入data
        for i in range(0, len(data), 2):
            DATA += self.normal_button_hex_dict[data[i:i + 2]]
        if len(DATA) < 8:
            DATA += b'\x00' * (8 - len(DATA))
        else:
            DATA = DATA[:8]

        # 分离HEAD中的值，并计算和
        HEAD_hex_list = []
        for byte in HEAD:
            HEAD_hex_list.append(byte)
        HEAD_add_hex_list = sum(HEAD_hex_list)

        # 分离DATA中的值，并计算和
        DATA_hex_list = []
        for byte in DATA:
            DATA_hex_list.append(byte)
        DATA_add_hex_list = sum(DATA_hex_list)

        #
        try:
            SUM = sum([HEAD_add_hex_list, int.from_bytes(ADDR, byteorder='big'),
                       int.from_bytes(CMD, byteorder='big'), int.from_bytes(LEN, byteorder='big'),
                       DATA_add_hex_list]) % 256  # 校验和
        except OverflowError:
            print("int too big to convert")
            return False
        packet = HEAD + ADDR + CMD + LEN + DATA + bytes([SUM])  # 数据包
        port.ser.write(packet)  # 将命令代码写入串口
        return True  # 如果成功，则返回True，否则引发异常

    """
    释放按钮。
    
    参数:
        serial (object): 用于发送数据的串行对象。
    
    返回:
        None
    """

    def release(serial):
        serial.send_data('')



import random
import time
import os
import json
import pyautogui as pyautogui
import serial
from . import BezierTrajectory


class DataComm:
    """
    属性：
        screen_width: 屏幕宽度
        screen_height: 屏幕高度
    """

    def __init__(self, screen_width=3840, screen_height=2160):
        self.hex_dict = {"ST": b'\x02',
                         "NU": b"\x00",
                         "LE": b"\x01",
                         "RI": b"\x02",
                         "CE": b"\x04"
                         }
        self.number_list = 0
        self.le = 0
        self.deviation = 0
        self.bias = 0
        self.type = 0
        self.X_MAX = screen_width
        self.Y_MAX = screen_height

    """
    发送数据到串口，将鼠标光标移动到屏幕上的绝对位置。
    
    参数：
        x (int)：鼠标光标的x坐标。
        y (int)：鼠标光标的y坐标。
        ctrl (str)：按下的控制键（可选）。
        port (serial)：要写入数据的串口（可选）。
    
    返回：
        如果成功发送数据，则返回True，否则返回False。
    """

    def send_data_absolute(self, x: int, y: int, ctrl: str = '', port: serial = serial) -> bool:
        # 将字符转写为数据包
        HEAD = b'\x57\xAB'  # 帧头
        ADDR = b'\x00'  # 地址
        CMD = b'\x04'  # 命令
        LEN = b'\x07'  # 数据长度
        DATA = bytearray(b'\x02')  # 数据

        # 鼠标按键
        if ctrl == '':
            DATA.append(0)
        elif isinstance(ctrl, int):
            DATA.append(ctrl)
        else:
            DATA += self.hex_dict[ctrl]

        # 坐标
        X_Cur = (4096 * x) // self.X_MAX
        Y_Cur = (4096 * y) // self.Y_MAX
        DATA += X_Cur.to_bytes(2, byteorder='little')
        DATA += Y_Cur.to_bytes(2, byteorder='little')

        if len(DATA) < 7:
            DATA += b'\x00' * (7 - len(DATA))
        else:
            DATA = DATA[:7]

        # 分离HEAD中的值，并计算和
        HEAD_hex_list = list(HEAD)
        HEAD_add_hex_list = sum(HEAD_hex_list)

        # 分离DATA中的值，并计算和
        DATA_hex_list = list(DATA)
        DATA_add_hex_list = sum(DATA_hex_list)

        try:
            SUM = sum([HEAD_add_hex_list, int.from_bytes(ADDR, byteorder='big'),
                       int.from_bytes(CMD, byteorder='big'), int.from_bytes(LEN, byteorder='big'),
                       DATA_add_hex_list]) % 256  # 校验和
        except OverflowError:
            print("int too big to convert")
            return False
        packet = HEAD + ADDR + CMD + LEN + DATA + bytes([SUM])  # 数据包
        port.ser.write(packet)  # 将命令代码写入串口
        return True  # 如果成功，则返回True，否则引发异常

    """
    发送相对于鼠标的数据。
    
    参数：
        x (int)：鼠标的x坐标。
        y (int)：鼠标的y坐标。
        ctrl (str)：按下的控制键。
        port (Serial)：鼠标的串口。
    
    返回：
        bool：如果成功发送数据，则返回True，否则返回False。
    """

    def send_data_relatively(self, x: int, y: int, ctrl: str = '', port: serial = serial) -> bool:
        # 将字符转写为数据包
        HEAD = b'\x57\xAB'  # 帧头
        ADDR = b'\x00'  # 地址
        CMD = b'\x05'  # 命令
        LEN = b'\x05'  # 数据长度
        DATA = bytearray(b'\x01')  # 数据

        # 鼠标按键
        if ctrl == '':
            DATA.append(0)
        elif isinstance(ctrl, int):
            DATA.append(ctrl)
        else:
            DATA += self.hex_dict[ctrl]

        # x坐标
        if x == 0:
            DATA.append(0)
        elif x < 0:
            DATA += (0 - abs(x)).to_bytes(1, byteorder='big', signed=True)
        else:
            DATA += x.to_bytes(1, byteorder='big', signed=True)

        # y坐标，这里为了符合坐标系直觉，将<0改为向下，>0改为向上
        y = - y
        if y == 0:
            DATA.append(0)
        elif y < 0:
            DATA += (0 - abs(y)).to_bytes(1, byteorder='big', signed=True)
        else:
            DATA += y.to_bytes(1, byteorder='big', signed=True)

        DATA += b'\x00' * (5 - len(DATA)) if len(DATA) < 5 else DATA[:5]

        # 分离HEAD中的值，并计算和
        HEAD_hex_list = list(HEAD)
        HEAD_add_hex_list = sum(HEAD_hex_list)

        # 分离DATA中的值，并计算和
        DATA_hex_list = list(DATA)
        DATA_add_hex_list = sum(DATA_hex_list)

        try:
            SUM = sum([HEAD_add_hex_list, int.from_bytes(ADDR, byteorder='big'),
                       int.from_bytes(CMD, byteorder='big'), int.from_bytes(LEN, byteorder='big'),
                       DATA_add_hex_list]) % 256  # 校验和
        except OverflowError:
            print("int too big to convert")
            return False
        packet = HEAD + ADDR + CMD + LEN + DATA + bytes([SUM])  # 数据包
        port.ser.write(packet)  # 将命令代码写入串口
        return True  # 如果成功，则返回True，否则引发异常
    
    """ 
    功能：move_to_basic
    
    描述：此函数调用轨迹生成函数生成的逐差数组，将数组中的数据发送到串口以控制鼠标移动。
    
    参数：
    - self：对象实例
    - x：目标的x坐标。
    - y：目标的y坐标。
    - ctrl：鼠标移动的控制字符（默认为空字符串）。
    - port：串口（默认为serial）。
    
    返回：
    - None
    """  
    
    def move_to_basic(self, x: int, y: int, ctrl: str = '', port: serial = serial) -> None:
        coordinate = [x, y]
        number_list = int((((x ** 2) + (y ** 2)) ** 0.5))
        le = random.randint(4, 10)
        deviation = random.randint(0, int(min(abs(x) % 499, abs(y) % 499) / 5))
        bias = random.uniform(0, 0.8)
        type_ = 3
        bezierTrajectory = BezierTrajectory.BezierTrajectory()
        bezier_array = bezierTrajectory.get_track([0, 0], coordinate, number_list, le, deviation, bias, type_)

        # test
        # print("====================参数信息=============================================")
        # print("x=%d y=%d number_list=%d le=%d deviation=%d bias=%f type=%d" % (
        #     x, y, number_list, le, deviation, bias, type_))

        # 将bezier_array数组转为整数
        for i in range(len(bezier_array)):
            bezier_array[i][0] = int(bezier_array[i][0])
            bezier_array[i][1] = int(bezier_array[i][1])

        # 定义prefix_bezier_array数组，存放差值
        prefix_bezier_array = [[0, 0]]
        for i in range(len(bezier_array) - 1):
            prefix_bezier_array.append(
                [bezier_array[i + 1][0] - bezier_array[i][0], bezier_array[i + 1][1] - bezier_array[i][1]])

        # 过滤prefix_bezier_array数组中的部分[0,0]元素
        # new_bezier_array = []
        # count = 0
        # for i in range(len(prefix_bezier_array)):
        #     if prefix_bezier_array[i] == [0, 0]:
        #         count += 1
        #         if count < 2:
        #             new_bezier_array.append(prefix_bezier_array[i])
        #     else:
        #         count = 0
        #         new_bezier_array.append(prefix_bezier_array[i])
        # prefix_bezier_array = new_bezier_array

        # 定义prefix_bezier_array数组，存放差值
        for i in range(len(prefix_bezier_array)):
            self.send_data_relatively(int(prefix_bezier_array[i][0]), int(prefix_bezier_array[i][1]), ctrl, port)
            time.sleep(0.0016)

    """
    这个函数检查鼠标指针在屏幕上的理论值和实际值之间的差异。
    
    参数：
    - self：MouseDataComm对象。
    - x：鼠标指针位置的x坐标。
    - y：鼠标指针位置的y坐标。
    
    返回：
    - difference_ratio：鼠标指针在屏幕上的理论值和实际值之间的差异。
    """

    def check_difference_ratio(self, x: int, y: int) -> float:
        # 创建一个MouseDataComm类的实例
        mouse = DataComm()
        # 调用send_data_absolute方法将鼠标指针移动到屏幕的中心
        mouse.send_data_absolute(int(self.X_MAX / 2), int(self.Y_MAX / 2))
        # 调用move_to方法，参数为(50,50)
        mouse.move_to_basic(x, y)
        # 计算鼠标的位置距离屏幕中心点的距离，并将距离输出到控制台上
        x_, y_ = pyautogui.position()
        distance = ((x_ - self.X_MAX / 2) ** 2 + (y_ - self.Y_MAX / 2) ** 2) ** 0.5
        difference_ratio = distance / (((x ** 2) + (y ** 2)) ** 0.5)
        print("理论值与实际值之间的差异：", (difference_ratio * 100), "%")
        return difference_ratio

    """
    功能：get_corrector
    
    描述：此函数从JSON文件中检索校正器值。如果文件不存在，则创建它并通过平均8个不同坐标的检查函数结果来计算校正器值。
    
    参数： 
    - self：对象实例
    
    返回：
    - value：校正器值
    """

    def get_corrector(self) -> float:
        file_path = os.path.join(os.getcwd(), 'corrector', 'information.json')
        if not os.path.exists(file_path):
            os.makedirs(os.path.dirname(file_path), exist_ok=True)
            with open(file_path, 'w') as file:
                data = {'value': 0}
                json.dump(data, file)
            value_sum = 0
            for x, y in [(5, 20), (50, 25), (233, 233), (313, 212), (500, 10), (500, 100), (17, 587),
                         (666, 666)]:
                value_sum += self.check_difference_ratio(x, y)
            value = value_sum / 8
            with open(file_path, 'w') as file:
                data = {'value': value}
                json.dump(data, file)
            return value
        else:
            with open(file_path, 'r') as file:
                data = json.load(file)
                value = data['value']
                return value

    """
    将鼠标光标移动到指定的目标坐标。

    参数：
        dest_x (int)：目标的x坐标。
        dest_y (int)：目标的y坐标。
        ctrl (str, optional)：要发送到串口的控制字符。默认为''。
        port (serial, optional)：要使用的串口。默认为serial。

    返回：
        None
    """
    
    def move_to(self, dest_x: int, dest_y: int, ctrl: str = '', port: serial = serial) -> None:
        start_x, start_y = pyautogui.position()
        correction_factor = self.get_corrector()
        corrected_x = dest_x * (1 / correction_factor)
        corrected_y = dest_y * (1 / correction_factor)
        self.move_to_basic(int(corrected_x), int(corrected_y), ctrl, port)
        end_x, end_y = pyautogui.position()
        distance = ((end_x - start_x) ** 2 + (end_y - start_y) ** 2) ** 0.5
        difference_ratio = distance / (((dest_x ** 2) + (dest_y ** 2)) ** 0.5)
        # test
        print("理论值与实际值之间的差异：", (difference_ratio * 100), "%")

    """
    单击鼠标左键。
    
    参数：
        serial (Serial)：串行对象。
    
    返回：
        None
    """

    def click(self, port: serial = serial) -> None:
        self.send_data_relatively(0, 0, 'LE', port)
        time.sleep(random.uniform(0.1, 0.45))  # 100到450毫秒延迟
        self.send_data_relatively(0, 0, 'NU', port)



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name="ch9329Comm",
    version="1.0.8",
    author="北极小狐",
    author_email="yuhao888123@gmail.com",
    description="提供CH9329芯片的键盘/鼠标串口快捷通信方法",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=['ch9329Comm'],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)



import math
import random
import numpy as np


class BezierTrajectory:

    @staticmethod
    def _bztsg(data_trajectory):
        length_of_data = len(data_trajectory)

        def staer(x):
            t = ((x - data_trajectory[0][0]) / (data_trajectory[-1][0] - data_trajectory[0][0]))
            y = np.array([0, 0], dtype=np.float64)
            for s in range(length_of_data):
                y += data_trajectory[s] * ((math.factorial(length_of_data - 1) // (
                        math.factorial(s) * math.factorial(length_of_data - 1 - s))) * math.pow(t, s) * math.pow(
                    (1 - t), length_of_data - 1 - s))
            return y[1]

        return staer

    def _type(self, type_, x, number_list):
        number_list_re = []
        pin = (x[1] - x[0]) / number_list
        if type_ == 0:
            for i in range(number_list):
                number_list_re.append(i * pin)
            if pin >= 0:
                number_list_re = number_list_re[::-1]
        elif type_ == 1:
            for i in range(number_list):
                number_list_re.append(1 * ((i * pin) ** 2))
            number_list_re = number_list_re[::-1]
        elif type_ == 2:
            for i in range(number_list):
                number_list_re.append(1 * ((i * pin - x[1]) ** 2))

        elif type_ == 3:
            data_trajectory = [np.array([0, 0]), np.array([(x[1] - x[0]) * 0.8, (x[1] - x[0]) * 0.6]),
                               np.array([x[1] - x[0], 0])]
            fun = self._bztsg(data_trajectory)
            number_list_re = [0]
            for i in range(1, number_list):
                number_list_re.append(fun(i * pin) + number_list_re[-1])
            if pin >= 0:
                number_list_re = number_list_re[::-1]
        number_list_re = np.abs(np.array(number_list_re) - max(number_list_re))
        biao_number_list = ((number_list_re - number_list_re[number_list_re.argmin()]) / (
                number_list_re[number_list_re.argmax()] - number_list_re[number_list_re.argmin()])) * (x[1] - x[0]) + x[
                               0]
        biao_number_list[0] = x[0]
        biao_number_list[-1] = x[1]
        return biao_number_list

    def simulation(self, start, end, le=1, deviation=0, bias=0.5):
        """

        :param start:开始点的坐标 如 start = [0, 0]
        :param end:结束点的坐标 如 end = [100, 100]
        :param le:几阶贝塞尔曲线，越大越复杂 如 le = 4
        :param deviation:轨迹上下波动的范围 如 deviation = 10
        :param bias:波动范围的分布位置 如 bias = 0.5
        :return:返回一个字典equation对应该曲线的方程，P对应贝塞尔曲线的影响点
        """
        start = np.array(start)
        end = np.array(end)
        cbb = []
        if le != 1:
            e = (1 - bias) / (le - 1)
            cbb = [[bias + e * i, bias + e * (i + 1)] for i in range(le - 1)]

        data_trajectory_list = [start]

        t = random.choice([-1, 1])
        w = 0
        for i in cbb:
            px1 = start[0] + (end[0] - start[0]) * (random.random() * (i[1] - i[0]) + (i[0]))
            p = np.array([px1, self._bztsg([start, end])(px1) + t * deviation])
            data_trajectory_list.append(p)
            w += 1
            if w >= 2:
                w = 0
                t = -1 * t

        data_trajectory_list.append(end)
        return {"equation": self._bztsg(data_trajectory_list), "P": np.array(data_trajectory_list)}

    def get_track(self, start, end, number_list, le=1, deviation=0, bias=0.5, type_=0, cbb=0, yhh=10):
        """

        :param start:开始点的坐标 如 start = [0, 0]
        :param end:结束点的坐标 如 end = [100, 100]
        :param number_list:返回的数组的轨迹点的数量 number_list = 150
        :param le:几阶贝塞尔曲线，越大越复杂 如 le = 4
        :param deviation:轨迹上下波动的范围 如 deviation = 10
        :param bias:波动范围的分布位置 如 bias = 0.5
        :param type_:0表示均速滑动，1表示先慢后快，2表示先快后慢，3表示先慢中间快后慢 如 type_ = 1
        :param cbb:在终点来回摆动的次数
        :param yhh:在终点来回摆动的范围
        :return:返回一个字典trackArray对应轨迹数组，P对应贝塞尔曲线的影响点
        """
        s = []
        fun = self.simulation(start, end, le, deviation, bias)
        w = fun['P']
        fun = fun["equation"]
        if cbb != 0:
            number_list_of_cbb = round(number_list * 0.2 / (cbb + 1))
            number_list -= (number_list_of_cbb * (cbb + 1))

            x_track_array = self._type(type_, [start[0], end[0]], number_list)
            for i in x_track_array:
                s.append([i, fun(i)])
            dq = yhh / cbb
            kg = 0
            ends = np.copy(end)
            for i in range(cbb):
                if kg == 0:
                    d = np.array([end[0] + (yhh - dq * i),
                                  ((end[1] - start[1]) / (end[0] - start[0])) * (end[0] + (yhh - dq * i)) + (
                                          end[1] - ((end[1] - start[1]) / (end[0] - start[0])) * end[0])])
                    kg = 1
                else:
                    d = np.array([end[0] - (yhh - dq * i),
                                  ((end[1] - start[1]) / (end[0] - start[0])) * (end[0] - (yhh - dq * i)) + (
                                          end[1] - ((end[1] - start[1]) / (end[0] - start[0])) * end[0])])
                    kg = 0
                print(d)
                y = self.get_track(ends, d, number_list_of_cbb, le=2, deviation=0, bias=0.5, type_=0, cbb=0, yhh=10)
                s += list(y['trackArray'])
                ends = d
            y = self.get_track(ends, end, number_list_of_cbb, le=2, deviation=0, bias=0.5, type_=0, cbb=0, yhh=10)
            s += list(y['trackArray'])

        else:
            x_track_array = self._type(type_, [start[0], end[0]], number_list)
            for i in x_track_array:
                s.append([i, fun(i)])
        return s
